/*      ldlas.c  

	Main program of the ldlas package
	(c) 1995, 1996 by Thor Sigvaldason
	
	This file contains the main() routine of ldlas 
	See the online help or included documentation for
	details of use.
	
*/

/* Some comprehensive definitions */

#define MAX_VARS	50	/* Maximum Number of Variables          */
#define MAX_SVARS	5	/* Maximum Number of String Variables   */
#define MAX_OBS		10000	/* Maximum Number of Observations       */                
#define MAX_CONS        256     /* Maximum Number of Constants          */
#define INT_PREC        100      /* Precision of integration calculations*/

/*	If you need to handle larger data sets than this, then just change the
	numbers above to suit your tastes (and recompile!). Note that the
	number of string variables is a subset of the total number of 
	variables. In other words, with 20 MAX_VAR's and 5 MAX_SVAR's, there 
	are only 15 ``vectors'' left for float and integer data.
	
*/


#define LDLAS_VER 0.8               /* Version Number of the Program        */
#define LDLAS_DATE "Jan. 1996     " /* Latest Date of Source Code           */
#define MAX_INPUT 256               /* Maximum characters in command string */
#define MAX_PARTS 100               /* Max commands in a command string     */
#define DESC_LENGTH 40              /* Characters in a variable's descrip.  */
				    /* and largest length of any entry in a */
				    /* string vector                        */
#define NAME_LENGTH 20              /* Characters in a variable's name      */
#define MAX_HISTORY 50              /* Number of recallable command lines   */
#define MIN_COLS 40                 /* Minimum number of Columns on terminal*/
				    /* required for ldlas to start          */
#define MIN_LINES 22                /* As above, but Rows                   */
#define READ 0                      /* For pipes and stdin                  */
#define WRITE 1                     /* For pipes and stdout                 */
#define ERROR 2                     /* For pipes and stderr                 */
#define DEF_LINEFORM 1              /* Default Line Style (Solid) in Graphs */
#define DEF_POINTFORM 1             /* Default Point Style (o) in Graphs    */
#define DEF_KEYFORM 1               /* Default Key Placement in Graphs      */
#define DEF_KEYSCALE 0.1            /* Default key scaling option           */
#define DEF_GRAPHFORM "postscript eps 'Helvetica' 18"
				    /* Default Graph Format*/

/* Do the includes. Note that the order of includes is important            */

#include <ncurses.h>    /* This is (obviously) for all basic i/o            */ 

WINDOW *ioarea;		/* The main i/o area window shared by all routines  */
WINDOW *statusbar;	/* Black bar at bottom to indicate status	    */


/* Below is the data structure for storing (vector of) variables            */

struct vector
	{
		char  v_name[NAME_LENGTH];
		char  v_desc[DESC_LENGTH];
		int   nobs;
		int   fnumb;
		int   inumb;
		int   snumb;
		double *fmem_start;
		int   *imem_start;
		char  *smem_start;
	};
struct single_vector
	{
		char  c_name[NAME_LENGTH];
		float c_value;
	};
struct  single_vector constants[MAX_CONS];        /*Storing of _constants   */
struct	vector d_matrix[MAX_VARS];                /*Organization of data    */
double	*fvec[MAX_VARS];                          /*Float variable matrix   */
int	*ivec[MAX_VARS];			  /*Integer variable matrix */
char	smatrix[MAX_SVARS][MAX_OBS][DESC_LENGTH]; /*String variable matrix  */
char	d_matrix_desc[DESC_LENGTH];		  /*Holds data descriptions */
int	existslog, existstex, ask_more;           /*			    */
int	oktoquit, escapeldlas, alert_time;	  /*  Some option flags     */
char	coinput[MAX_INPUT];			  /*Command string variable */
char	cohistory[MAX_HISTORY][MAX_INPUT];        /*Command history matrix  */
int	missing_ob;				  /*Value for missing obser */
FILE	*log_name;				  /*File pointer for log    */
FILE	*tex_name;				  /*File pointer for tex log*/
FILE	*batch_name;				  /*File pointer for batch  */
int     astart;					  /*Start obser for loops   */
int     astop;					  /*End obser for loops     */
double  working[MAX_OBS];			  /*A vector for sorting    */
int     worksize;                                 /*Size of sorting vector  */
char    graphform[MAX_INPUT];			  /*Graph output format     */
int     lineform;				  /*Default line style      */
int     pointform;				  /*Default point style     */
int     keyform;				  /*Default key location    */
float   keyscale;				  /*Scaling option for key  */
int     batchmode;                                /*Are we batching?        */
char    batchname[MAX_INPUT];			  /*Batch file              */
int     screwup;				  /*Flag for passing errors */
int     inreglist[MAX_VARS];                      /*Flag for regression     */
float   reglist[MAX_VARS];                        /*Regression coefficient  */
float   theintercept;                             /*Intercept value         */

extern void small_error(int enumber);

#include "stdlib.h"     /* Should compile just about anywhere               */
#include "math.h"       /* Need fab() and sqrt()                            */
#include "setstate.h"   /* Does init ( startup() ), and shutdown()  )       */
#include "manip.h"      /* Routines for string (ie. command) manipulations  */
#include "errors.h"	/* error(n) returns a string of error message n     */
#include "displays.h"   /* for the list and show commands              	    */
#include "ldataio.h"    /* Data input and output                            */
#include "helpinfo.h"   /* Help and other info modules                      */
#include "stats.h"      /* Standard statistical routines                    */
#include "dobatch.h"    /* Batch language and related commands              */
#include "regress.h"    /* Regression and related commands                  */
#include "interact.h"   /* Shell commands and gnuplot interaction           */

void bye(void);

/* main() gets user input, cparse()'s it, and then calls relevant  routines */

main( )
{
	int countclear;
	char waisted[MAX_INPUT][MAX_INPUT];
	oktoquit=TRUE;
	escapeldlas=0;
	missing_ob=-999999;
	screwup=FALSE;
	alert_time = 10; /* Default time for alert and quote boxes          */
	ask_more=TRUE;   /* Scrolling displays will pause after each screen */
	existslog=FALSE; /* No log file by default			    */
	existstex=FALSE; /* No tex log file by default			    */
	strcpy(graphform,DEF_GRAPHFORM);
	lineform=DEF_LINEFORM;
	pointform=DEF_POINTFORM;
	keyform=DEF_KEYFORM;
	keyscale=DEF_KEYSCALE;
	batchmode=FALSE;
	for (countclear = 0; countclear < MAX_VARS; countclear++)
	{
		d_matrix[countclear].fnumb = -1;
		d_matrix[countclear].inumb = -1;
		d_matrix[countclear].snumb = -1;
		inreglist[countclear] = FALSE;
	}
	for (countclear = 0; countclear < MAX_CONS; countclear++)
	{
		constants[countclear].c_value = 0;
		strcpy(constants[countclear].c_name, "\0");
	}
	if(startup() == FALSE) escapeldlas = 1;
	do
	{
		if(escapeldlas == 0)
		{
			if(batchmode == FALSE)
			{
				loutput("\nldlas>");
				wrefresh(ioarea);
				saystatus("Command Mode      ");
				get_input(coinput);
			}
			else
			{
				batch_input(coinput);
			}
			switch (parse_input(puller(coinput)))
			{
				case 1	: bye();		break;
				case 2  : help(coinput);	break;
				case 3  : unmangle();		break;
				case 4  : warranty();		break;
				case 5  : why();		break;
				case 6  : about();		break;
				case 7  : d_input(coinput);	break;
				case 8  : d_list(coinput);	break;
				case 9	: break;
				case 10 : describe(); 		break;
				case 11 : showstate(); 		break;
				case 12 : ddrop(coinput); 	break;
				case 13 : label(coinput); 	break;
				case 14 : dsave(coinput); 	break;
				case 15 : dload(coinput); 	break;
				case 16 : finput(coinput);	break;
				case 17 : summary(coinput);	break;
				case 18 : display_time();	break;
				case 19 : set(coinput);		break;
				case 20 : alert(randmessage(),alert_time); break;
				case 21 : dlog(coinput);	break;
				case 22 : dtlog(coinput);       break;
				case 23 : ldwait();	        break;
				case 25 : ldecho(coinput);	break;
				case 26 : alert(error(28),alert_time);	break;
				case 27 : interp(coinput);	break;
				case 28 : alert(error(30),alert_time);	break;
				case 29 : alert(error(31),alert_time);	break;
				case 30 : ls_dir(coinput);	break;
				case 31 : alert(error(29),alert_time);  break;
				case 32 : dobatch(coinput);	break;
				case 33 : fappend(coinput);	break;
				case 34 : histogram(coinput);	break;
				case 35 : ldreplace(coinput);	break;
				case 36 : ldappend(coinput);	break;
				case 39 : regress(coinput);	break;
				case 41 : ldgen(coinput);	break;
				case 42 : ldpredict(coinput);	break;
				case 43 : ldrename(coinput);	break;
				case 47 : ldgraph(coinput);     break;
				case 48 : ldlet(coinput);	break;
				case 49 : ldcalc(coinput);	break;
				case 99 : small_error(1); 	break;
			}
		
		}
	}
	while(escapeldlas==0);
	shutdown();
}

void bye(void)
{
	if(oktoquit == TRUE) escapeldlas = 1;
	else alert(error(1), alert_time);
}
	